import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;

public class NewTCPgui {

    final static int width = 400;
    final static int height = 400;
    JFrame topFrame;
    JLabel textLabel;
    final JTextField messageField;
    final JTextArea displayArea;

    final PrintStream ps;
    final BufferedReader br;

    NewTCPgui (final PrintStream ps, final BufferedReader br,
               String title, boolean goFirst) {
        // Set the stream and the reader.
        this.ps = ps;
        this.br = br;

        // Create the GUI.
        topFrame = new JFrame(title);
        Container contentPane = topFrame.getContentPane();

        displayArea = new JTextArea("");
        displayArea.setEditable(false);
        JScrollPane sp = new JScrollPane(displayArea);

        textLabel = new JLabel("Enter message, hit return: ");
        messageField = new JTextField(20);

        JPanel panel = new JPanel();
        panel.add(textLabel);
        panel.add(messageField);

        contentPane.add(panel, BorderLayout.NORTH);
        contentPane.add(sp, BorderLayout.CENTER);

        // The bulk of the work is done in the anonymous
        // ActionListener when the user hits return.
        messageField.addActionListener (new ActionListener() {
            public void actionPerformed (ActionEvent e) {
                // Get the message to transmit.
                String message = e.getActionCommand();
                System.out.println(">" + message + "\n");
                // Clear the message field
                messageField.setText("");
                // Make the message field non-editable.
                messageField.setEditable (false);
                // Transmit the message.
                ps.println(message);
                ps.flush();
                // Append the message to the message area.
                displayArea.append (">" + message + "\n");
                // Check if the program should terminate.
                if (message.equals("")) {
                    System.out.println("Ending chat.");
                    System.exit (0);
                }
                // Reading a reply is done in a separate thread, so
                // the GUI thread is free to update the GUI.
                new ReplyReader().start();
                // Continue chatting.
            }
        });

        // Handle closing the window explicitly.
        topFrame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                    System.out.println("Window closing");
                    System.exit (0);
                }
        });

        // If not going first, don't allow to send a message
        // and wait to recieve a message.
        if (!goFirst) {
            messageField.setEditable(false);
            new ReplyReader().start();
        }

        topFrame.setSize(width, height);
        topFrame.setVisible(true);
    }

    class ReplyReader extends Thread {
        public void run() {
          try {
              // Read from the connection.
              String inString = br.readLine();
              // Check if the other side has terminated.
              if (inString == null ||
                  inString.equals("")) {
                  System.out.println("Chat ended.");
                  System.exit(0);
              }
              // Append the reply to the display area.
              displayArea.append("<" + inString + "\n");
              System.out.println("<" + inString + "\n");
              // Make the message field editable so
              // a new message can be sent.
              messageField.setEditable(true);
          }
          catch (IOException ioe) {
              ioe.printStackTrace();
              System.out.println(
               "Error when reading a new line: " + ioe);
          }
        }
    }
}